MODULE iceini
   !!======================================================================
   !!                       ***  MODULE iceini   ***
   !!   Sea-ice model : LIM Sea ice model Initialization
   !!======================================================================
#if defined key_lim3
   !!----------------------------------------------------------------------
   !!   'key_lim3' :                                   LIM sea-ice model
   !!----------------------------------------------------------------------
   !!   ice_init       : sea-ice model initialization
   !!----------------------------------------------------------------------
   USE dom_oce
   USE in_out_manager
   USE ice_oce         ! ice variables
   USE flx_oce
   USE phycst          ! Define parameters for the routines
   USE ocfzpt
   USE ice
   USE limmsh
   USE limistate
   USE limrst
   USE par_ice
   USE limvar
   USE limicepoints

   IMPLICIT NONE
   PRIVATE

   !! * Routine accessibility
   PUBLIC ice_init                 ! called by opa.F90
   PUBLIC lim_itd_ini

   !! * Share Module variables
   LOGICAL , PUBLIC  ::     & !!! ** init namelist (namicerun) **
      ln_limdyn   = .TRUE., & !: flag for ice dynamics (T) or not (F)
      ln_nicep    = .TRUE.    !: flag for sea-ice points output (T) or not (F)
   INTEGER , PUBLIC  ::   &  !:
      nstart ,            &  !: iteration number of the begining of the run 
      nlast  ,            &  !: iteration number of the end of the run 
      nitrun ,            &  !: number of iteration
      numit                  !: iteration number
   REAL(wp), PUBLIC  ::   &  !:
      hsndif = 0.e0    ,  &  !: computation of temp. in snow (0) or not (9999)
      hicdif = 0.e0    ,  &  !: computation of temp. in ice (0) or not (9999)
      tpstot           ,  &  !: time of the run in seconds
      cai    = 1.40e-3 ,  &  !: atmospheric drag over sea ice
      cao    = 1.00e-3       !: atmospheric drag over ocean
   REAL(wp), PUBLIC, DIMENSION(2)  ::  &  !:
      acrit  = (/ 1.e-06 , 1.e-06 /)    !: minimum fraction for leads in 
      !                                   !  north and south hemisphere
   !!----------------------------------------------------------------------
   !!   LIM 2.0,  UCL-LOCEAN-IPSL (2005) 
   !! $Header: /home/opalod/NEMOCVSROOT/NEMO/LIM_SRC/iceini.F90,v 1.4 2005/03/27 18:34:41 opalod Exp $ 
   !! This software is governed by the CeCILL licence see modipsl/doc/NEMO_CeCILL.txt 
   !!----------------------------------------------------------------------

CONTAINS

   SUBROUTINE ice_init
      !!----------------------------------------------------------------------
      !!                  ***  ROUTINE ice_init  ***
      !!
      !! ** purpose :   
      !!
      !! History :
      !!   8.5  !  02-08  (G. Madec)  F90: Free form and modules
      !!----------------------------------------------------------------------

      INTEGER :: ji,jj,jk,jl, index

      ! Open the namelist file 
      OPEN( numnam_ice, FILE= 'namelist_ice', FORM='formatted', STATUS = 'old') 

      CALL ice_run                    !  read in namelist some run parameters

      CALL lim_icepoints              !  define specific checking ice points
                 
      ! Louvain la Neuve Ice model
      IF( nacc == 1 ) THEN
          dtsd2   = nfice * rdtmin * 0.5
          rdt_ice = nfice * rdtmin
      ELSE
          dtsd2   = nfice * rdt * 0.5
          rdt_ice = nfice * rdt
      ENDIF

      CALL lim_msh                    ! ice mesh initialization
     
      CALL lim_itd_ini
      ! Initial sea-ice state
      IF( .NOT.ln_rstart ) THEN
         numit = 0
         ! martin ajoute
         numit = nit000 - 1
         CALL lim_istate              ! start from rest: sea-ice deduced from sst
         CALL lim_var_agg(1)
         CALL lim_var_glo2eqv
      ELSE
         CALL lim_rst_read( numit )   ! start from a restart file
         ! martin ajoute
         numit = nit000 - 1
         CALL lim_var_agg(1)
         CALL lim_var_glo2eqv
      ENDIF

      freeze(:,:) = at_i(:,:)   ! initialisation of sea/ice cover    
# if defined key_coupled
      alb_ice(:,:) = albege(:,:)      ! sea-ice albedo
# endif
      
      nstart = numit  + nfice      
      nitrun = nitend - nit000 + 1 
      nlast  = numit  + nitrun 

      IF( nstock == 0  )  nstock = nlast + 1

   END SUBROUTINE ice_init

   SUBROUTINE ice_run
      !!-------------------------------------------------------------------
      !!                  ***  ROUTINE ice_run ***
      !!                 
      !! ** Purpose :   Definition some run parameter for ice model
      !!
      !! ** Method  :   Read the namicerun namelist and check the parameter 
      !!       values called at the first timestep (nit000)
      !!
      !! ** input   :   Namelist namicerun
      !!
      !! history :
      !!   2.0  !  03-08 (C. Ethe)  Original code
      !!-------------------------------------------------------------------
      NAMELIST/namicerun/ ln_limdyn, acrit, hsndif, hicdif, cai, cao, ln_nicep
      !!-------------------------------------------------------------------

      !                                           ! Read Namelist namicerun 
      REWIND ( numnam_ice )
      READ   ( numnam_ice , namicerun )
      IF(lwp) THEN
         WRITE(numout,*)
         WRITE(numout,*) 'ice_run : ice share parameters for dynamics/advection/thermo of sea-ice'
         WRITE(numout,*) ' ~~~~~~'
         WRITE(numout,*) '   switch for ice dynamics (1) or not (0)      ln_limdyn   = ', ln_limdyn
         WRITE(numout,*) '   minimum fraction for leads in the NH (SH)  acrit(1/2)   = ', acrit(:)
         WRITE(numout,*) '   computation of temp. in snow (=0) or not (=9999) hsndif = ', hsndif
         WRITE(numout,*) '   computation of temp. in ice  (=0) or not (=9999) hicdif = ', hicdif
         WRITE(numout,*) '   atmospheric drag over sea ice                           = ', cai
         WRITE(numout,*) '   atmospheric drag over ocean                             = ', cao
         WRITE(numout,*) '   Several ice points in the ice or not in ocean.output = ', ln_nicep
      ENDIF
      
   END SUBROUTINE ice_run

   SUBROUTINE lim_itd_ini
        !!------------------------------------------------------------------
        !!                ***  ROUTINE lim_itd_ini ***
        !! ** Purpose :
        !!            Initializes the ice thickness distribution
        !! ** Method  :
        !!            Very simple
        !!
        !! ** Arguments :
        !!           kideb , kiut : Starting and ending points on which the
        !!                         the computation is applied
        !!
        !! ** Inputs / Ouputs : (global commons)
        !!
        !! ** External :
        !!
        !! ** References :
        !!
        !! ** History :
        !!           (12-2005) Martin Vancoppenolle
        !!           Rien n'est jamais acquis
        !!           A l'homme ni sa force
        !!           Ni sa faiblesse ni son coeur
        !!           Et quand il croit
        !!           Ouvrir ses bras son ombre
        !!           Est celle d'une croix
        !!
        !!------------------------------------------------------------------
        !! * Arguments

       !! * Local variables
       INTEGER ::   ji,       &   ! spatial dummy loop index
                    jl,       &   ! ice category dummy loop index
                    jm            ! ice types    dummy loop index

       REAL(wp)  ::           &  ! constant values
          zeps      =  1.0e-10,   & !
          zc1                 ,   & !
          zc2                 ,   & !
          zc3                 ,   & !
          zx1

       WRITE(numout,*) 'lim_itd_ini : Initialization of ice thickness distribution '
       WRITE(numout,*) '~~~~~~~~~~~~~~~'

!!-- End of declarations
!!----------------------------------------------------------------------------------------------

!--------------------------------------------------------------------------------------------------!
! 1) Ice thickness distribution parameters initialization                                          !
!--------------------------------------------------------------------------------------------------!

      !- Types boundaries in integer thickness space
      !----------------------------------------------
      !- Type 1 (undeformed ice) Boundaries
      ice_cat_bounds(1,1) = 1
      ice_cat_bounds(1,2) = jpl
!     !- Type 2 (ridged ice  ) Boundaries
!     ice_cat_bounds(2,1) = 4
!     ice_cat_bounds(2,2) = 5
!     !- Type 3 (rafted ice  ) Boundaries
!     ice_cat_bounds(3,1) = 6
!     ice_cat_bounds(3,2) = 6

      !- Number of ice thickness categories in each ice type
      DO jm = 1, jpm
         ice_ncat_types(jm) = ice_cat_bounds(jm,2) - ice_cat_bounds(jm,1) + 1 
      END DO

      !- Make the correspondence between thickness categories and ice types
      !---------------------------------------------------------------------
      !- ice_types = 1 -> undeformed ice
      !- ice_types = 2 -> ridged   ice
      !- ice_types = 3 -> rafted   ice
      DO jm = 1, jpm       !over types
         DO jl = ice_cat_bounds(jm,1), ice_cat_bounds(jm,2) !over thickness categories
            ice_types(jl) = jm
         END DO
      END DO

      WRITE(numout,*) ' Number of ice types jpm =      ', jpm
      WRITE(numout,*) ' Number of ice categories jpl = ', jpl
      DO jm = 1, jpm
         WRITE(numout,*) ' Ice type ', jm
         WRITE(numout,*) ' Number of thickness categories ', ice_ncat_types(jm)
         WRITE(numout,*) ' Thickness category boundaries  ', ice_cat_bounds(jm,1:2)
      END DO
      WRITE(numout,*) 'Ice type vector', ice_types(1:jpl)
      WRITE(numout,*)

      !- Thickness categories boundaries in thickness space
      !---------------------------------------------------------------------

      ! NEW DEFINITION

         hi_max(:) = 0.0
         hi_max_typ(:,:) = 0.0

! new categories
!     !- Type 3 - rafted ice
!        hi_max(ice_cat_bounds(3,2)) = 5.0

!     !- Type 2 - ridged ice
!        zc1 = 3./REAL(ice_cat_bounds(2,2)-ice_cat_bounds(2,1)+1)
!        zc2 = 10.0*zc1
!        zc3 = 3.0

!        DO jl = ice_cat_bounds(2,1),ice_cat_bounds(2,2)
!           zx1 = REAL(jl-ice_cat_bounds(2,1)+1) / REAL(ice_cat_bounds(2,2)-ice_cat_bounds(2,1)+1)
!           hi_max(jl) = hi_max(jl-1) + zc1 + zc2 * (1.0 + TANH ( zc3 * (zx1 - 1.0 ) ) )
!        END DO

!        ! force the thickness into the ice categories
!        hi_max(ice_cat_bounds(2,1)) = 15.0
!        hi_max(ice_cat_bounds(2,2)) = 25.0

      !- Type 1 - undeformed ice
         zc1 = 3./REAL(ice_cat_bounds(1,2)-ice_cat_bounds(1,1)+1)
         zc2 = 10.0*zc1
         zc3 = 3.0

         DO jl = ice_cat_bounds(1,1), ice_cat_bounds(1,2)
            zx1 = REAL(jl-1) / REAL(ice_cat_bounds(1,2)-ice_cat_bounds(1,1)+1)
            hi_max(jl) = hi_max(jl-1) + zc1 + zc2 * (1.0 + TANH ( zc3 * (zx1 - 1.0 ) ) )
         END DO

      !- Fill in the hi_max_typ vector, useful in other circumstances
         DO jl = ice_cat_bounds(1,1), ice_cat_bounds(1,2)
            hi_max_typ(jl,1) = hi_max(jl)
         END DO
!        DO jl = ice_cat_bounds(2,1), ice_cat_bounds(2,2)
!           hi_max_typ(jl-ice_cat_bounds(2,1)+1,2) = hi_max(jl)
!        END DO
!        DO jl = ice_cat_bounds(3,1), ice_cat_bounds(3,2)
!           hi_max_typ(jl-ice_cat_bounds(3,1)+1,3) = hi_max(jl)
!        END DO

         WRITE(numout,*) ' Thickness category boundaries independently of ice type '
         WRITE(numout,*) ' hi_max ', hi_max(0:jpl)

         WRITE(numout,*) ' Thickness category boundaries inside ice types '
         DO jm = 1, jpm
            WRITE(numout,*) ' Type number ', jm
            WRITE(numout,*) ' hi_max_typ : ', hi_max_typ(0:ice_ncat_types(jm),jm)
         END DO

         DO jl = 1, jpl
            hi_mean(jl) = ( hi_max(jl) + hi_max(jl-1) ) / 2.0
         END DO

         tn_ice(:,:,:) = t_su(:,:,:)

    END SUBROUTINE lim_itd_ini

#else
   !!----------------------------------------------------------------------
   !!   Default option :        Empty module           NO LIM sea-ice model
   !!----------------------------------------------------------------------
CONTAINS
   SUBROUTINE ice_init        ! Empty routine
   END SUBROUTINE ice_init

   SUBROUTINE lim_itd_ini
   END SUBROUTINE lim_itd_ini
#endif

   !!======================================================================
END MODULE iceini
